using UnityEngine;
using System.Collections;


public class GameManager : MonoBehaviour {

    private static GameManager instance;
    private static GameObject container;

    public CameraController camera1;
    public GameObject player;
    public GameObject target;

    public Player playerscript;

    public int enemyHealth;

    float introTime;
    int introStage;

    float timeSinceEnd = 0f;

    GameObject music;
    float musicVol = 1;
    bool fading = false;

    public enum gameState
    {
        stateIntro,
        stateActive
    };

    public int state;

    public static GameManager GetInstance()
    {
        if (!instance)
        {
            container = new GameObject();
            container.name = "GameManager";
            instance = container.AddComponent(typeof(GameManager)) as GameManager;
        }
        return instance;
    }

	// Use this for initialization
	void Start () {
        if (DifficultyManager.GetInstance().skipIntro == false)
            state = (int)GameManager.gameState.stateIntro;  //Change this to stateActive to skip the intro.
        else
        {
            state = (int)GameManager.gameState.stateActive;
            startMusic();
        }

        camera1 = (CameraController)Camera.mainCamera.GetComponent("CameraController");

        player = GameObject.Find("playership");
        target = GameObject.Find("badtower");

        playerscript = GameObject.Find("PlayerController").GetComponent<Player>();

        introTime = 0.0f;

        if (state == (int)gameState.stateIntro)
        {
            player.transform.position = new Vector3(54f, 65f, -900f);
            introStage = 0;

        }

        enemyHealth = 750;
	}

    void OnGUI()
    {
        //This text displays after the game ends.
        if (enemyHealth <= 0)
        {
            timeSinceEnd += Time.deltaTime;
            if (timeSinceEnd > 18f)
            {
                if (playerscript.timesHit > 0)
                {
                    float width = 400f;
                    float xpos = Screen.width / 2f - width / 2f;
                    float ypos = Screen.height - 85f;

                    GUI.Box(new Rect(xpos, ypos, width, 40), "Thanks for Playing!\nTry and see if you can do it again without being hit!");

                    if (GUI.Button(new Rect(Screen.width / 2f - 100 / 2f, ypos + 45f, 100f, 25f), "Back to Menu"))
                    {
                        Time.timeScale = 1f; //Because you can click the button before slowdown ends on boss death.
                        Application.LoadLevel(0);
                    }
                }
                else
                {
                    float width = 400f;
                    float xpos = Screen.width / 2f - width / 2f;
                    float ypos = Screen.height - 85f;

                    if(DifficultyManager.GetInstance().gameDifficulty < 3)
                        GUI.Box(new Rect(xpos, ypos-15, width, 55), "Great Job!\nYou cleared this difficulty without being hit!\nSee if you can do it on a higher difficulty level!");
                    else
                        GUI.Box(new Rect(xpos, ypos-15, width, 55), "Flawless!\nYou beat the hardest difficulty without a single mistake!\nYou really are amazing!");

                    if (GUI.Button(new Rect(Screen.width / 2f - 100 / 2f, ypos + 50f, 100f, 25f), "Back to Menu"))
                    {
                        Time.timeScale = 1f; //Because you can click the button before slowdown ends on boss death.
                        Application.LoadLevel(0);
                    }
                }

            }
        }
        else
        {
            if (state == (int)GameManager.gameState.stateActive)
            {
                switch (DifficultyManager.GetInstance().gameDifficulty)
                {
                    case 0:
                        GUI.Box(new Rect(Screen.width - 135, 10, 125, 25), "Easy Difficulty");
                        break;
                    case 1:
                        GUI.Box(new Rect(Screen.width - 135, 10, 125, 25), "Normal Difficulty");
                        break;
                    case 2:
                        GUI.Box(new Rect(Screen.width - 135, 10, 125, 25), "Hard Difficulty");
                        break;
                    case 3:
                        GUI.Box(new Rect(Screen.width - 135, 10, 125, 25), "Lunatic Difficulty");
                        break;
                    default:
                        GUI.Box(new Rect(Screen.width - 135, 10, 125, 25), "Unknown Difficulty");
                        break;
                }
            }
        }
    }


    public void DamageTarget(int damage)
    {
        enemyHealth -= damage;
    }

    public void startMusic()
    {
        //Why is the music code in the camera controller?  WHO KNOWS!
        music = (GameObject)GameObject.Instantiate(Resources.Load("Music"), Vector3.zero, Quaternion.identity);
    }

    IEnumerator fadeMusic()
    {
        fading = true;
        if(music) //Incase you skip the intro
            music.audio.volume = musicVol;

        yield return null;

        musicVol -= Time.deltaTime/10;

        if (musicVol < 0)
            musicVol = 0;
        else
            StartCoroutine(fadeMusic());    
        
    }
	
	// Update is called once per frame
	void Update () {

        // Dhast Intro Scene!  It's super fancy!  Seriously!

        if (state == (int)gameState.stateIntro)
        {
            camera1.height = 130;

            iTween.MoveTo(player, iTween.Hash("z", -50, "time", 18f, "easetype", "easeOutSine", "name", "player"));

            if (introStage < 2)
                camera1.FollowPlayerOnly();

            if (introTime > 2f && introStage == 0)
            {
                GameObject.Instantiate(Resources.Load("WarningSound"), Vector3.zero, Quaternion.identity);
                StartCoroutine(camera1.flashWarning(3));
                introStage = 1;
            }

            if (introTime > 6f && introStage == 1)
            {
                StartCoroutine(camera1.showBossName(4.5f));
                introStage = 2;
            }

            if (introStage == 2)
            {
                iTween.RotateTo(camera1.gameObject, iTween.Hash("x", -20, "easetype", "easeOutQuint", "time", 6));
                iTween.MoveTo(camera1.gameObject, iTween.Hash("position", new Vector3(54f,20f,-60f), "easetype", "easeOutQuint", "time", 4));
            }

            if (introTime > 13 && introStage == 2)
                introStage = 3;

            if (introStage == 3)
            {
                Vector3 lookfrom = (target.transform.position + new Vector3(player.transform.position.x, 200f, -50f)) * 0.5f;
                lookfrom.y = 200;

                iTween.RotateTo(camera1.gameObject, iTween.Hash("x", 90, "easetype", "easeOutQuint", "time", 4));
                iTween.MoveTo(camera1.gameObject, iTween.Hash("position", lookfrom, "easetype", "easeOutQuint", "time", 4, "name", "cammove"));
            }

            if (introTime > 17.5f)
            {
                state = (int)gameState.stateActive;
                camera1.height = 200;
                iTween.StopByName(player, "player");
                iTween.StopByName(camera1.gameObject, "cammove");
            }

            introTime += Time.deltaTime;
        }

        if (enemyHealth <= 0 && !fading)
            StartCoroutine(fadeMusic());

        if (enemyHealth > 0)
            Time.timeScale = 1f;
	}
}
