using UnityEngine;
using System.Collections;

public class Player : MonoBehaviour {

    public float horizontalLimit = 52;
    public float verticalLimit = 42;
    public float playerSpeed = 40;
    public float fireDelay = 0.3f;

    public GameObject player;
    public GameObject target;

    DifficultyManager dm;

    BulletManager bm;
    GameManager gm;
    float lastShotTime = 0;
    int lastGunFired = 0;

    GameObject[] guns = new GameObject[3];

    public int timesHit = 0;
    float lastHit;

    public int playerState;

    enum PlayerStatus
    {
        playerAlive,
        playerDead,
        playerRespawning
    }

    public void TakeDamage()
    {
        if (Time.time - lastHit > 2f)
        {
            timesHit++;
            lastHit = Time.time;

            TowerBulletBasic bscript;

            GameObject.Instantiate(Resources.Load("Detonator-Insanity"), player.gameObject.transform.position, Quaternion.identity);

            foreach (GameObject bullet in GameObject.FindGameObjectsWithTag("Bullet"))
            {
                bscript = bullet.GetComponent<TowerBulletBasic>();
                StartCoroutine(bscript.DieAfterDistance(player.gameObject));
            }
        }
    }

    void OnGUI()
    {
        if (gm.state == (int)GameManager.gameState.stateActive && timesHit > 0)
            GUI.Box(new Rect(10, 40, 125, 25), "Times Hit: " + timesHit);
    }

	// Use this for initialization
    void Start ()
    {
        //set start position
        player.transform.position = new Vector3(54f, 65f, -50f);

        //Attach bullet manager and game manager
        bm = BulletManager.GetInstance();
        gm = GameManager.GetInstance();
        dm = DifficultyManager.GetInstance();

        //Load our guns.
        guns[0] = GameObject.Find("/playership/Ship/Gun1");
        guns[1] = GameObject.Find("/playership/Ship/Gun2");
        guns[2] = GameObject.Find("/playership/Ship/Gun3");

        playerState = (int)Player.PlayerStatus.playerAlive;
    }
	
	// Update is called once per frame
	void Update () 
    {
        float xMove = Input.GetAxisRaw("Horizontal") * playerSpeed * Time.deltaTime;
        float zMove = Input.GetAxisRaw("Vertical") * playerSpeed * Time.deltaTime;

        
        if (gm.state == (int)GameManager.gameState.stateActive && playerState != (int)Player.PlayerStatus.playerDead)
        {
            //player movement!

            player.transform.position += new Vector3(xMove, 0f, zMove);

            //Point at the cursor

            if (dm.autoAim == false)
            {
                Plane shipPlane = new Plane(Vector3.up, player.transform.position);

                Ray raycast = Camera.main.ScreenPointToRay(Input.mousePosition);

                float hitDist = 0.0f;

                if (shipPlane.Raycast(raycast, out hitDist))
                {
                    Vector3 mouseCursor = raycast.GetPoint(hitDist);

                    Quaternion lookAt = Quaternion.LookRotation(mouseCursor - player.transform.position);

                    player.transform.rotation = Quaternion.Lerp(player.transform.rotation, lookAt, 5f * Time.deltaTime);
                }
            }
            else
            {
                if (!target && gm.enemyHealth > 0)
                {
                    //We lost target.  Lets try to re-get it.

                    target = GameObject.Find("TowerTop");
                }

                if (target || gm.enemyHealth > 0)
                {
                    Vector3 enemyLoc = new Vector3(target.transform.position.x, 65f, target.transform.position.z);

                    Quaternion lookAt = Quaternion.LookRotation(enemyLoc - player.transform.position);

                    player.transform.rotation = Quaternion.Lerp(player.transform.rotation, lookAt, 5f * Time.deltaTime);
                }
            }

            //It's Firing Time

            lastShotTime = Mathf.Clamp(lastShotTime - Time.deltaTime, 0, fireDelay);

            if (Input.GetButton("Fire1") && lastShotTime <= 0)
            {
                lastShotTime = fireDelay;

                bm.CreateBullet("PlayerShot", guns[lastGunFired].transform.position, guns[0].transform.rotation, 150, 3f);
                lastGunFired++;
                if (lastGunFired > 2)
                    lastGunFired = 0;
            }

            //Limit the playfield if the game is active

            if (gm.state == (int)GameManager.gameState.stateActive)
            {
                if (player.transform.position.x > 210)
                    player.transform.position = new Vector3(210f, player.transform.position.y, player.transform.position.z);
                if (player.transform.position.x < -100)
                    player.transform.position = new Vector3(-100f, player.transform.position.y, player.transform.position.z);
                if (player.transform.position.z < -120)
                    player.transform.position = new Vector3(player.transform.position.x, player.transform.position.y, -120f);
                if (player.transform.position.z > 160)
                    player.transform.position = new Vector3(player.transform.position.x, player.transform.position.y, 160f);
            }
        }
	}
}
