using UnityEngine;
using System.Collections;

public class TowerManager : MonoBehaviour {

    GameObject towerTop;
    GameObject towerGun1;
    GameObject towerGun2;
    GameObject towerGun3;
    GameObject towerGun4;

    GameObject player;

    GameObject laser1;
    GameObject laser2;
    GameObject laser3;
    GameObject laser4;

    GameObject spikeCollider;

    GameObject newtower;

    BulletManager bm;
    GameManager gm;

    float spinspeed = 0.6f;
    float turntimer = 0f;
    float direction = 1;

    bool isActive = false;
    int activephase;
    int subphase;

    int difficulty;
    bool laserstate = true;

    int firePhase = 0;
    int firePhase2 = 0;

	// Use this for initialization
    void Start () 
    {
        towerTop = GameObject.Find("/badtower/TowerArmature/TowerTop");
        towerGun1 = GameObject.Find("/badtower/TowerArmature/TowerTop/Gun1");
        towerGun2 = GameObject.Find("/badtower/TowerArmature/TowerTop/Gun2");
        towerGun3 = GameObject.Find("/badtower/TowerArmature/TowerTop/Gun3");
        towerGun4 = GameObject.Find("/badtower/TowerArmature/TowerTop/Gun4");

        bm = BulletManager.GetInstance();
        gm = GameManager.GetInstance();

        player = GameObject.Find("playership");

        difficulty = DifficultyManager.GetInstance().gameDifficulty;
    }

    public IEnumerator Phase1()
    {
        float diffmod = 1f - (0.25f * difficulty);

        float yieldtime = diffmod - spinspeed / 5f;

        switch(subphase)
        {
            case 0:
                if (yieldtime < 0.2f + (0.4f * diffmod))
                    yieldtime = 0.2f + (0.4f * diffmod);
                if (difficulty == 3)
                    yieldtime = 0.15f;
                break;
            case 1:
                if (yieldtime < 0.1f + (0.2f * diffmod))
                    yieldtime = 0.1f + (0.2f * diffmod);
                if (difficulty == 3)
                    yieldtime = 0.08f;
                break;
        }


        yield return new WaitForSeconds(yieldtime);

        float shotspeed = 20f + spinspeed * 10;
        if (shotspeed > 70)
            shotspeed = 70;
        if (subphase == 1)
            shotspeed = 100;
        float life = 200 / shotspeed;

        bm.CreateBullet("TowerBasic", towerGun1.transform.position, towerGun1.transform.rotation, shotspeed, life);
        bm.CreateBullet("TowerBasic", towerGun2.transform.position, towerGun2.transform.rotation, shotspeed, life);
        bm.CreateBullet("TowerBasic", towerGun3.transform.position, towerGun3.transform.rotation, shotspeed, life);
        bm.CreateBullet("TowerBasic", towerGun4.transform.position, towerGun4.transform.rotation, shotspeed, life);

        GameObject.Instantiate(Resources.Load("TowerShotSound"), Vector3.zero, Quaternion.identity);

        firePhase++;

        if (gm.enemyHealth < 700 && firePhase > 3 && difficulty > 1)
        {
            //The tower top isn't actually level with the player so we need to take the tower top's position, with the players height.
            Vector3 towertoppos = new Vector3(towerTop.transform.position.x, 62, towerTop.transform.position.z);

            Quaternion fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f,-90f,0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, shotspeed / 2, life * 2);

            if (difficulty == 3)
            {
                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, -60f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, shotspeed / 2, life * 2);

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, -120f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, shotspeed / 2, life * 2);
            }
            firePhase = 0;
        }

        if (gm.enemyHealth > 500 && gm.enemyHealth <= 625 && subphase == 0)
        {
            GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun1.transform.position, Quaternion.identity);
            GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun2.transform.position, Quaternion.identity);
            GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);
            GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun4.transform.position, Quaternion.identity);

            spinspeed += 5;
            subphase++;
        }
        if (gm.enemyHealth > 525)
        {
            StartCoroutine(Phase1());
        }
        if (gm.enemyHealth <= 525)
        {
            StartCoroutine(MidPhase());
        }
    }

    public IEnumerator MidPhase()
    {
        spinspeed -= 5 * Time.deltaTime;

        yield return null;

        if (spinspeed < 0 && activephase == 1)
        {
            spinspeed = 0;
            activephase = 2;
            subphase = 0;
            StartCoroutine(Phase2());
        }
        else
            StartCoroutine(MidPhase());
    }

    public IEnumerator Phase2()
    {
        if (subphase == 0)
        {
            laser1 = (GameObject)GameObject.Instantiate(Resources.Load("TowerLaser"), Vector3.zero, Quaternion.identity);
            laser1.transform.parent = towerGun1.transform;
            laser1.transform.localScale = new Vector3(1f, 1f, 1f);
            laser1.transform.localPosition = new Vector3(0f, 0f, 0f);
            laser1.transform.localRotation = Quaternion.identity;

            laser2 = (GameObject)GameObject.Instantiate(Resources.Load("TowerLaser"), Vector3.zero, Quaternion.identity);
            laser2.transform.parent = towerGun2.transform;
            laser2.transform.localScale = new Vector3(1f, 1f, 1f);
            laser2.transform.localPosition = new Vector3(0f, 0f, 0f);
            laser2.transform.localRotation = Quaternion.identity;

            laser3 = (GameObject)GameObject.Instantiate(Resources.Load("TowerLaser"), Vector3.zero, Quaternion.identity);
            laser3.transform.parent = towerGun3.transform;
            laser3.transform.localScale = new Vector3(1f, 1f, 1f);
            laser3.transform.localPosition = new Vector3(0f, 0f, 0f);
            laser3.transform.localRotation = Quaternion.identity;

            laser4 = (GameObject)GameObject.Instantiate(Resources.Load("TowerLaser"), Vector3.zero, Quaternion.identity);
            laser4.transform.parent = towerGun4.transform;
            laser4.transform.localScale = new Vector3(1f, 1f, 1f);
            laser4.transform.localPosition = new Vector3(0f, 0f, 0f);
            laser4.transform.localRotation = Quaternion.identity;
            
            GameObject.Instantiate(Resources.Load("LaserSound"), Vector3.zero, Quaternion.identity);

            subphase = 1;
        }

        //Fire angled bullets!

        Quaternion aimat;
        Vector3 gunaim;

        float shotspeed = 40;
        if (difficulty == 3)
            shotspeed = 60;

        if (difficulty > 0)
        {
            gunaim = towerTop.transform.position - towerGun1.transform.position;
            aimat = Quaternion.LookRotation(Quaternion.Euler(0, -55, 0) * gunaim);
            bm.CreateBullet("TowerBasic", towerGun1.transform.position, aimat, shotspeed, 10);
            aimat = Quaternion.LookRotation(Quaternion.Euler(0, -125, 0) * gunaim);
            bm.CreateBullet("TowerBasic", towerGun1.transform.position, aimat, shotspeed, 10);

            gunaim = towerTop.transform.position - towerGun2.transform.position;
            aimat = Quaternion.LookRotation(Quaternion.Euler(0, -55, 0) * gunaim);
            bm.CreateBullet("TowerBasic", towerGun2.transform.position, aimat, shotspeed, 10);
            aimat = Quaternion.LookRotation(Quaternion.Euler(0, -125, 0) * gunaim);
            bm.CreateBullet("TowerBasic", towerGun2.transform.position, aimat, shotspeed, 10);

            gunaim = towerTop.transform.position - towerGun3.transform.position;
            aimat = Quaternion.LookRotation(Quaternion.Euler(0, -55, 0) * gunaim);
            bm.CreateBullet("TowerBasic", towerGun3.transform.position, aimat, shotspeed, 10);
            aimat = Quaternion.LookRotation(Quaternion.Euler(0, -125, 0) * gunaim);
            bm.CreateBullet("TowerBasic", towerGun3.transform.position, aimat, shotspeed, 10);

            gunaim = towerTop.transform.position - towerGun4.transform.position;
            aimat = Quaternion.LookRotation(Quaternion.Euler(0, -55, 0) * gunaim);
            bm.CreateBullet("TowerBasic", towerGun4.transform.position, aimat, shotspeed, 10);
            aimat = Quaternion.LookRotation(Quaternion.Euler(0, -125, 0) * gunaim);
            bm.CreateBullet("TowerBasic", towerGun4.transform.position, aimat, shotspeed, 10);

            GameObject.Instantiate(Resources.Load("TowerShotSound"), Vector3.zero, Quaternion.identity);
        }

        firePhase++;
        if (difficulty == 3 && firePhase > 3)
        {
            //The tower top isn't actually level with the player so we need to take the tower top's position, with the players height.
            Vector3 towertoppos = new Vector3(towerTop.transform.position.x, 62, towerTop.transform.position.z);
            Quaternion fireAtPlayer;

            fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 0f, 0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

            fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 45f, 0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

            fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 90f, 0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

            fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 135f, 0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

            fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 180f, 0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

            fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 225f, 0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

            fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 270f, 0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

            fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 315f, 0f));
            bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

            firePhase = 0;
        }


        //Time.timeScale = 0.5f;
        firePhase2++;
        if (difficulty == 3 && firePhase2 > 4)
        {
            if (laserstate == false)
            {
                laser1.SendMessage("SetStatus", false);
                laser2.SendMessage("SetStatus", false);
                laser3.SendMessage("SetStatus", false);
                laser4.SendMessage("SetStatus", false);
            }
            else
            {
                laser1.SendMessage("SetStatus", true);
                laser2.SendMessage("SetStatus", true);
                laser3.SendMessage("SetStatus", true);
                laser4.SendMessage("SetStatus", true);
            }

            laserstate = !laserstate;
            firePhase2 = 0;
        }

        float yieldtime = (gm.enemyHealth - 300f) / 100f + 0.5f;  //increase shot speed as health decreases.

        if (difficulty == 3)
            yieldtime = 0.5f;


        yield return new WaitForSeconds(yieldtime);

        if (activephase == 2)
            StartCoroutine(Phase2());

        //yield return null;
    }

    public IEnumerator Phase3Start()
    {
        activephase = 3;
        subphase = 0;

        if (laser1)
        {
            //LaserController lctrl = laser1.GetComponent<LaserController>();
            GameObject.Destroy(laser1);
            GameObject.Destroy(laser2);
            GameObject.Destroy(laser3);
            GameObject.Destroy(laser4);
        }

        iTween.RotateTo(towerTop,iTween.Hash("x", 0f, "islocal", true));

        GameObject oldtower = GameObject.Find("/badtower");

        GameObject.Instantiate(Resources.Load("Detonator-MushroomCloud"), oldtower.transform.position, Quaternion.identity);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun1.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun4.transform.position, Quaternion.identity);

        yield return new WaitForSeconds(0.5f);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun2.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);

        yield return new WaitForSeconds(0.5f);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerTop.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);


        Vector3 oldtransform = oldtower.transform.position;
        GameObject.Destroy(oldtower);

        newtower = (GameObject)GameObject.Instantiate(Resources.Load("badtower_flybit"), oldtransform, Quaternion.identity);
        GameObject deadtower = (GameObject)GameObject.Instantiate(Resources.Load("badtower_dead"), oldtransform, Quaternion.identity);

        CameraController camera = Camera.mainCamera.gameObject.GetComponent<CameraController>();
        camera.ChangeTargetObject(newtower);


        newtower.transform.localScale = new Vector3(3f, 3f, 3f);
        deadtower.transform.localScale = new Vector3(3f, 3f, 3f);

        towerTop = GameObject.Find("/badtower_flybit(Clone)/TowerArmature/TowerTop");
        towerGun1 = GameObject.Find("/badtower_flybit(Clone)/TowerArmature/TowerTop/Gun1");
        towerGun2 = GameObject.Find("/badtower_flybit(Clone)/TowerArmature/TowerTop/Gun2");
        towerGun3 = GameObject.Find("/badtower_flybit(Clone)/TowerArmature/TowerTop/Gun3");
        towerGun4 = GameObject.Find("/badtower_flybit(Clone)/TowerArmature/TowerTop/Gun4");

        towerTop.tag = "Enemy";
        
        SphereCollider sc = towerTop.AddComponent<SphereCollider>();

        sc.radius = 1.5f;
        sc.isTrigger = true;

        yield return new WaitForSeconds(10f);

        subphase = 1;

        spikeCollider = (GameObject)GameObject.Instantiate(Resources.Load("SpikeCollider"), Vector3.zero, Quaternion.identity);
        spikeCollider.transform.parent = towerTop.transform;
        spikeCollider.transform.localPosition = Vector3.zero;
        spikeCollider.transform.localScale = new Vector3(1f, 1f, 1f);

        StartCoroutine(Phase3());
        StartCoroutine(Phase3Shots());

        yield return null;
    }

    public IEnumerator Phase3()
    {
        //Tower flies at player with this

        float flytime = 3.5f;

        if (difficulty == 0)
            flytime = 5.5f;

        if (difficulty == 3)
            flytime = 1.5f;

        iTween.MoveTo(newtower, iTween.Hash("position", new Vector3(gm.player.transform.position.x, 0f, gm.player.transform.position.z), "time", flytime, "easetype", "easeInOutQuad"));

        if (difficulty >= 2)
        {
            //Hard and lunatic are just cruel.

            firePhase++;

            if (firePhase > (3 - difficulty))
            {
                //The tower top isn't actually level with the player so we need to take the tower top's position, with the players height.
                Vector3 towertoppos = new Vector3(towerTop.transform.position.x, 62, towerTop.transform.position.z);
                Quaternion fireAtPlayer;

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 0f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 45f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 90f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 135f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 180f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 225f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 270f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

                fireAtPlayer = Quaternion.LookRotation(towertoppos - player.transform.position) * Quaternion.Euler(new Vector3(0f, 315f, 0f));
                bm.CreateBullet("TowerBasicBig", towertoppos, fireAtPlayer, 35, 10);

                firePhase = 0;
            }
        }

        yield return new WaitForSeconds(flytime);

        if(gm.enemyHealth > 0)
            StartCoroutine(Phase3());
        yield return null;
    }

    public IEnumerator Phase3Shots()
    {
        float shotspeed = 40f;
        if (difficulty == 3)
            shotspeed = 60f;

        bm.CreateBullet("TowerBasic", towerGun1.transform.position, towerGun1.transform.rotation, shotspeed, 10);
        bm.CreateBullet("TowerBasic", towerGun2.transform.position, towerGun2.transform.rotation, shotspeed, 10);
        bm.CreateBullet("TowerBasic", towerGun3.transform.position, towerGun3.transform.rotation, shotspeed, 10);
        bm.CreateBullet("TowerBasic", towerGun4.transform.position, towerGun4.transform.rotation, shotspeed, 10);

        GameObject.Instantiate(Resources.Load("TowerShotSound"), Vector3.zero, Quaternion.identity);

        //faster, faster!  Shoot faster!  It seems like a trend.
        float delay = gm.enemyHealth / 200f;
        if (delay < 0.2f)
            delay = 0.2f;

        float diffmod = 1.5f - difficulty * 0.25f;
        if (difficulty == 0)
            delay += 0.2f;

        if (difficulty == 3)
            delay = delay * 0.5f + 0.05f;

        yield return new WaitForSeconds(delay * diffmod);

        if (gm.enemyHealth > 0)
            StartCoroutine(Phase3Shots());
        yield return null;
    }

    void OnGUI()
    {
        if (gm.state == (int)GameManager.gameState.stateActive && gm.enemyHealth > 0)
            GUI.Box(new Rect(10, 10, 125, 25), "Boss Health: " + gm.enemyHealth);
    }

    public IEnumerator Dying()
    {
        iTween.Stop(); //Stop the tower from chasing the player.

        spikeCollider.SendMessage("Destroy");
        
        newtower.animation.Play("die");
        Time.timeScale = 0.5f;  //Slowww moootttiooonnnnn

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun1.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun4.transform.position, Quaternion.identity);

        yield return new WaitForSeconds(0.5f);

        //Clear bullets in a sexy and dramatic way
        TowerBulletBasic bscript;

        foreach (GameObject bullet in GameObject.FindGameObjectsWithTag("Bullet"))
        {
            bscript = bullet.GetComponent<TowerBulletBasic>();
            StartCoroutine(bscript.DieAfterDistance(towerTop.gameObject));
        }

        yield return new WaitForSeconds(0.5f);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun2.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);

        yield return new WaitForSeconds(1f);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerTop.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);

        yield return new WaitForSeconds(0.3f);

        GameObject.Instantiate(Resources.Load("Detonator-MushroomCloud"), newtower.transform.position, Quaternion.identity);

        yield return new WaitForSeconds(1.1f);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun2.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerTop.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);

        GameObject.Instantiate(Resources.Load("Detonator-MushroomCloud"), newtower.transform.position, Quaternion.identity);

        yield return new WaitForSeconds(0.3f);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun2.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);

        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerTop.transform.position, Quaternion.identity);
        GameObject.Instantiate(Resources.Load("Detonator-Insanity"), towerGun3.transform.position, Quaternion.identity);

        yield return new WaitForSeconds(1f);
        
        GameObject.Destroy(newtower);


        yield return new WaitForSeconds(3f);

        Time.timeScale = 1f; // Normal motion

    }
	
	// Update is called once per frame
    void Update ()
    {
        float realspin = spinspeed;

        //This reverses the spin of the tower every so often.
        if (gm.state == (int)GameManager.gameState.stateActive && activephase < 3)
        {
            if (activephase == 0)
                activephase = 1;
            spinspeed += Time.deltaTime / 5 / activephase;
            if (spinspeed > 20)
                spinspeed = 20;
            turntimer += Time.deltaTime;

            if (activephase == 2 && difficulty == 3)
                turntimer = 0f;

            float turntime = 8f - spinspeed / 2f;
            if (turntime < 3f)
                turntime = 3f;

            if (turntimer > turntime)
            {
                turntimer = 0f;
                direction *= -1f;
            }

            if (direction < 0)
            {
                realspin = realspin - turntimer * realspin;
                if (realspin < spinspeed * -1)
                    realspin = spinspeed * -1;
            }
            else
            {
                realspin = (realspin * -1) + turntimer * realspin;

                if (realspin > spinspeed)
                    realspin = spinspeed;
            }
        }

        if (activephase == 2 && difficulty == 3)
        {
            //Lunatic laser max spin speed.
            realspin *= 2;
            if (realspin > 3f)
                realspin = 3f;
            if (realspin < -3f)
                realspin = -3f;
        }
        
        //force it to spin at a set speed when it goes into spike mode
        if (activephase == 3)
            realspin = 20;

        if (activephase == 2 && difficulty <= 1)
            realspin *= 0.5f;

        //Spin the tower.  Make sure it's alive if we're going to spin it or weird things happen.
        if (gm.enemyHealth > 0)
            towerTop.transform.RotateAround(Vector3.up, realspin * Time.deltaTime * 0.3f);

        //Start spike phase
        if (gm.enemyHealth < 300 && activephase == 2)
            StartCoroutine(Phase3Start());

        //The tower is dead, death animation time
        if (gm.enemyHealth <= 0 && activephase == 3 && subphase == 1)  //subphase1 check makes sure the tower completed transition to phase 3 before dying
        {
            StartCoroutine(Dying());
            activephase = 4;
        }

        if (!isActive && gm.state == (int)GameManager.gameState.stateActive)
        {
            StartCoroutine(Phase1());
            activephase = 1;
            subphase = 0;
            spinspeed = 5;
            isActive = true;
        }

    }
}
